const {KafkaClient, Producer, KeyedMessage} = require('kafka-node');
const R = require('ramda');
const debug = require('debug')('post-json-to-kafka:debug');
const trace = require('debug')('post-json-to-kafka:trace');

const stringifyMessageTopic_ = (to, objectMessage) => R.pipe(
  R.assoc('to', to),
  JSON.stringify
)(objectMessage);

const formatAllMessages_ = (to, messages) => R.map(
  message => stringifyMessageTopic_(to, message),
  messages
);

const getPayloadBase_ = (to, topic, messages) => R.pipe(
  () => R.objOf('messages', formatAllMessages_(to, messages)),
  R.assoc('topic', topic)
)();

const getPayload_ = (to, topic, messages) => R.pipe(
  () => getPayloadBase_(to, topic, messages),
  R.of,
  R.tap(debug)
)();

const getPayloadWithKey_ = (to, topic, messages, objectKeyPartition) => R.pipe(
  () => getPayloadBase_(to, topic, messages),
  R.over(R.lensProp('messages'), R.map(
    message => new KeyedMessage(R.prop('key', objectKeyPartition), message)
  )
  ),
  R.assoc('key', R.prop('key', objectKeyPartition)),
  R.assoc('partition', R.prop('partition', objectKeyPartition)),
  R.of,
  R.tap(debug)
)();

const getOptionPartitionType_ = type => R.ifElse(
  () => R.has(type, Producer.PARTITIONER_TYPES),
  R.always(R.objOf('partitionerType', R.prop(type, Producer.PARTITIONER_TYPES))),
  R.always({})
)();

const sendToKafka =
  (topic, to, messages, partitionerType = 'default') =>
    new Promise((resolve, reject) => {
      trace('Topic : ', topic);
      trace('to : ', to);
      trace('messages : ', messages);
      const client = new KafkaClient({kafkaHost: process.env.KAFKA_URL});
      const producer = new Producer(client, getOptionPartitionType_(partitionerType));

      producer.on('ready', () => {
        debug('Producer ready');
        producer.send(getPayload_(to, topic, messages), () => {
          debug('Payload sent');
          client.close(resolve);
        });
      });
      producer.on('error', error =>
        reject(error));
    });

const sendToKafkaWithKey =
  (topic, to, messages, objectKeyPartition, partitionerType = 'keyed',) =>
    new Promise((resolve, reject) => {
      trace('Topic : ', topic);
      trace('to : ', to);
      trace('messages : ', messages);
      trace('key : ', R.prop('key', objectKeyPartition));
      trace('partition : ', R.prop('partition', objectKeyPartition));
      const client = new KafkaClient({kafkaHost: process.env.KAFKA_URL});
      const producer = new Producer(client, getOptionPartitionType_(partitionerType)
      );

      producer.on('ready', () => {
        debug('Producer ready');
        producer.send(getPayloadWithKey_(to, topic, messages, objectKeyPartition), () => {
          debug('Payload sent');
          client.close(resolve);
        });
      });
      producer.on('error', error =>
        reject(error));
    });

module.exports = {sendToKafka, sendToKafkaWithKey};
